@@ -14,21 +14,21 @@ setMethod(f="fitEnsemble",
             method="EM",
             exp=numeric(),
             useModelParams = TRUE,
-            predType="posteriorMedian",
+            predType="posteriorMean",
             const=0,
             W = rep(1/dim(.forecastData@predCalibration)[2],dim(.forecastData@predCalibration)[2]),
             iterations= 40000,
             burns = 20000,
             thinning = 20,
-            modelPriors = rep(1/dim(.forecastData@predCalibration)[2],dim(.forecastData@predCalibration)[2])
+            modelPriors = rep(1,dim(.forecastData@predCalibration)[2])
             )
           {
             if(iterations < (burns)){
               stop("Number of iterations is smaller than the burnin. Increase the number of iterations or decrease the burnin.")
             }
-            if(method == "gibbs" & any(is.na(.forecastData@predCalibration))){
-              stop("Missing values in the calibration set are currently only allowed with EM estimation.")
-            }
+            #if(method == "gibbs" & any(is.na(.forecastData@predCalibration))){
+            #  stop("Missing values in the calibration set are currently only allowed with EM estimation.")
+            #}
             if(method == "gibbs"){
               cat("Model weights estimated using gibbs sampling")
             }
@@ -128,11 +128,14 @@ setMethod(f="fitEnsemble",
             if(useModelParams==TRUE){
               .models <- plyr::alply(predCalibration, 2:3, .fun=.modelFitter)
               for(i in 1:nMod){
-                if(any(unname(.models[[i]][[2]]) > 0.5)){
-                  cat("WARNING: Problematic Cook's Distances (> 0.5) \n", "Model", names(.models[i]), ":",
-                      which(unname(.models[[i]][[2]]) > 0.5), "\n")
+                if(any(unname(.models[[i]][[2]]) > 0.5, na.rm = TRUE)){
+                  cat("WARNING: Problematic Cook's Distances (> 0.5) for", "Model", names(.models[i]), "\n")
+                      #which(unname(.models[[i]][[2]]) > 0.5), "\n")
                   #warning("Problematic Cook's Distances (> 0.5), see above output (under 'this.ensemble').")
                 }
+                if(all(is.finite(unname(.models[[i]][[2]]))==FALSE)){
+                  cat("WARNING: Cook's Distances could not be calculated for", "Model", names(.models[i]), "\n Probably too few non-missing observations. \n")
+                }
                 .models[[i]] <- .models[[i]][[1]]
               }
             }
@@ -217,22 +220,22 @@ setMethod(f="fitEnsemble",
               .flatPreds <- plyr::aaply(predCalibrationAdj, c(1,2), function(x) {mean(x, na.rm=TRUE)})
               .sdVec <- rep(sqrt(sigma2), nMod)
   
-              if (predType=="posteriorMean"){
+              #if (predType=="posteriorMean"){
                 bmaPred <- array(plyr::aaply(.flatPreds, 1, function(x) {sum(x* W, na.rm=TRUE)}), dim=c(nObsCal, 1,nDraws))
                 bmaPred <-  bmaPred/array(t(W%*%t(1*!is.na(.flatPreds))), dim=c(nObsCal, 1, nDraws))
                 bmaPred[,,-1] <- NA
-              }
+              #}
   
-              if (predType=="posteriorMedian"){
-                .altQBMAnormal <- function(x){
-                  .x <- x[!is.na(x)]
-                  .W <- W[!is.na(x)]
-                  ..sdVec <- .sdVec[!is.na(x)]
-                  .ebmaMedian(.W, .x, ..sdVec)
-                }
-               bmaPred <- array(plyr::aaply(.flatPreds, 1, .altQBMAnormal),  dim=c(nObsCal, 1,nDraws))
-               bmaPred[,,-1] <- NA
-              }
+              #if (predType=="posteriorMedian"){
+              #  .altQBMAnormal <- function(x){
+              #    .x <- x[!is.na(x)]
+              #    .W <- W[!is.na(x)]
+              #    ..sdVec <- .sdVec[!is.na(x)]
+              #    .ebmaMedian(.W, .x, ..sdVec)
+              #  }
+              # bmaPred <- array(plyr::aaply(.flatPreds, 1, .altQBMAnormal),  dim=c(nObsCal, 1,nDraws))
+              # bmaPred[,,-1] <- NA
+              #}
               cal <- abind::abind(bmaPred, .forecastData@predCalibration, along=2); colnames(cal) <- c("EBMA", modelNames)
               
               if(sum(W)<=.99 || sum(W)>1.01){
@@ -245,7 +248,12 @@ setMethod(f="fitEnsemble",
               
               ### if W is not a matrix, call to rcpp for em
               if(is.null(dim(W))){
-                out  = GibbsNormal(outcomeCalibration, matrix(predCalibrationAdj[,,1],ncol=nMod), W, modelPriors, sigma2, iterations, burns, thinning)
+                if(any(is.na(.forecastData@predCalibration))){
+                  out  = GibbsNormalMissing(outcomeCalibration, matrix(predCalibrationAdj[,,1],ncol=nMod), W, modelPriors, sigma2, iterations, burns, thinning)
+                }
+                if(any(is.na(.forecastData@predCalibration))==F){
+                  out  = GibbsNormal(outcomeCalibration, matrix(predCalibrationAdj[,,1],ncol=nMod), W, modelPriors, sigma2, iterations, burns, thinning)
+                }
                 W.mat <- out$W
                 Sigma.mat = out$Sigma
               }
@@ -254,14 +262,19 @@ setMethod(f="fitEnsemble",
               # checking to see if the weights significantly differ
               if(is.matrix(W)){
                 # Matrix to store all posterior weights in
-                store.W.median <- matrix(data=NA, nrow=dim(W)[1], ncol=dim(W)[2])
-                colnames(store.W.median) <- modelNames
+                store.W.mean <- matrix(data=NA, nrow=dim(W)[1], ncol=dim(W)[2])
+                colnames(store.W.mean) <- modelNames
                 
                 for(i in 1:dim(W)[1]){
                   vectorW <- W[i,]
-                  out  = GibbsNormal(outcomeCalibration, matrix(predCalibrationAdj[,,1],ncol=nMod), vectorW, alpha = modelPriors, sigma = sigma2, iterations, burns, thinning)
+                  if(any(is.na(.forecastData@predCalibration))){
+                    out  = GibbsNormalMissing(outcomeCalibration, matrix(predCalibrationAdj[,,1],ncol=nMod), vectorW, modelPriors, sigma2, iterations, burns, thinning)
+                  }
+                  if(any(is.na(.forecastData@predCalibration))==F){
+                    out  = GibbsNormal(outcomeCalibration, matrix(predCalibrationAdj[,,1],ncol=nMod), vectorW, modelPriors, sigma2, iterations, burns, thinning)
+                  }
                   W.mat <- out$W
-                  store.W.median[i,] <- apply(W.mat, 2, FUN=median)
+                  store.W.mean[i,] <- apply(W.mat, 2, FUN=mean)
                   }
                 ### save posterior weights based on last set of initial weights
                 W.mat <- W.mat
@@ -271,11 +284,11 @@ setMethod(f="fitEnsemble",
                 combs <- gtools::combinations(dim(W)[1], 2)
                 store.MAD <- rep(NA,dim(combs)[1])
                 for(i in 1:dim(combs)[1]){
-                  store.MAD[i] <- mean(abs(store.W.median[combs[i,1], ] - store.W.median[combs[i,2], ]), na.rm = T)
+                  store.MAD[i] <- mean(abs(store.W.mean[combs[i,1], ] - store.W.mean[combs[i,2], ]), na.rm = T)
                   }
                 # Error if any mean absolute difference of posterior weights > 0.0001
-                if(any(store.MAD > 0.0001)){
-                  cat("WARNING: The mean absolute difference between the sets of median posterior weights is above 0.0001. \n
+                if(any(store.MAD > 0.001)){
+                  cat("WARNING: The mean absolute difference between the sets of mean posterior weights is above 0.001. \n
                           The posterior EBMA prediction is only based on the last set of weights.")
                   }
               }
@@ -324,22 +337,22 @@ setMethod(f="fitEnsemble",
               .flatPredsTest <- matrix(plyr::aaply(predTestAdj, c(1,2), function(x) {mean(x, na.rm=TRUE)}), ncol=nMod)
               
               if(method == "EM"){
-                if (predType=="posteriorMean"){
+                #if (predType=="posteriorMean"){
                   bmaPredTest <-array(plyr::aaply(.flatPredsTest, 1, function(x) {sum(x* W, na.rm=TRUE)}), dim=c(nObsTest, 1,nDraws))
                   bmaPredTest <-  bmaPredTest/array(t(W%*%t(1*!is.na(.flatPredsTest))), dim=c(nObsTest, 1, nDraws))
                   bmaPredTest[,,-1] <- NA
-                }
+                #}
   
-                if (predType=="posteriorMedian"){
-                  .altQBMAnormal <- function(x){
-                    .x <- x[!is.na(x)]
-                    .W <- W[!is.na(x)]
-                    ..sdVec <- .sdVec[!is.na(x)]
-                    .ebmaMedian( .W, .x, ..sdVec)
-                  }
-                  bmaPredTest <- array(plyr::aaply(.flatPredsTest, 1, .altQBMAnormal),  dim=c(nObsTest, 1,nDraws))
-                  bmaPredTest[,,-1] <- NA
-                }
+                #if (predType=="posteriorMedian"){
+                #  .altQBMAnormal <- function(x){
+                #    .x <- x[!is.na(x)]
+                #    .W <- W[!is.na(x)]
+                #    ..sdVec <- .sdVec[!is.na(x)]
+                #    .ebmaMedian( .W, .x, ..sdVec)
+                #  }
+                #  bmaPredTest <- array(plyr::aaply(.flatPredsTest, 1, .altQBMAnormal),  dim=c(nObsTest, 1,nDraws))
+                #  bmaPredTest[,,-1] <- NA
+                #}
   
                 test <- abind::abind(bmaPredTest, .forecastData@predTest, along=2);  colnames(test) <- c("EBMA", modelNames)
                 }
